@@ -1,6 +1,6 @@
 # import os
 # import unicodecsv as csv
-from modules import csvExporter
+import csvExporter
 
 # from datetime import datetime
 # from modules.global_variables import EXPORT_PATH, ACTIONS_EXPORT_FILENAME, CONFIG_NAME
