@@ -1,12 +1,23 @@
 import argparse
 import re
+
+import questionary as questionary
 import yaml
+from questionary import Separator
 from yaml.scanner import ScannerError
 
 from modules.global_variables import *
+from modules.last_successful import get_last_successful, parse_ls, update_sync_marker_file
 from modules.logger import log_critical_error, create_directory_if_not_exists
 # from safetypy import safetypy as sp
 import modules.safetypy as sp
+from modules.setup_questions import questions, flat_config, model_config
+from modules.sql import test_sql_settings
+
+from pprint import pp
+from rich import print, box
+from rich.panel import Panel
+
 
 def load_setting_api_access_token(logger, config_settings):
     """
@@ -229,13 +240,13 @@ def set_env_defaults(name, env_var, logger):
         if name == 'CONFIG_NAME':
             logger.error('You must set the CONFIG_NAME')
             sys.exit()
-        if name == 'DB_SCHEMA':
+        elif name == 'DB_SCHEMA':
             env_var = 'dbo'
-        if name.startswith('DB_'):
+        elif name.startswith('DB_'):
             env_var = None
-        if name == 'SQL_TABLE':
+        elif name == 'SQL_TABLE':
             env_var = None
-        if name == 'TEMPLATE_IDS':
+        elif name == 'TEMPLATE_IDS':
             env_var = None
         else:
             env_var = 'false'
@@ -284,7 +295,7 @@ def load_actions_table(actions_table_name):
         return actions_table_name
 
 
-def load_config_settings(logger, path_to_config_file, docker_enabled):
+def load_config_settings(logger, path_to_config_file, docker_enabled=False):
     """
     Load config settings from config file
 
@@ -495,7 +506,29 @@ def parse_command_line_arguments(logger):
                                                              'Directory will be named iAuditor Audit Exports, and will '
                                                              'be placed in your current directory')
     args = parser.parse_args()
-    print(args)
+
+    if args.setup:
+        logo = '''
+                               *****                      
+                             *******##(                   
+                          ********#######                 
+                        *******     ########              
+                     ********         *#####              
+                  ,*******                                
+                ********                      *****       
+               ******                       *******.      
+                                         ********         
+                        ///            ********           
+                       //////*      ********              
+                        ////////  ********                
+                           /////*******                   
+                             *******,                     
+
+              Thanks for using the iAuditor Exporter!'''
+        print(Panel(logo, box=box.SIMPLE, expand=False))
+        initial_setup(logger)
+        exit()
+
     if args.docker is None:
         if args.config is None:
             rename_config_sample(logger)
@@ -513,6 +546,8 @@ def parse_command_line_arguments(logger):
     else:
         config_filename = None
 
+
+
     export_formats = ['pdf']
     if args.format is not None and len(args.format) > 0:
         valid_export_formats = ['json', 'docx', 'pdf', 'csv', 'media', 'web-report-link', 'actions', 'actions-sql',
@@ -528,7 +563,278 @@ def parse_command_line_arguments(logger):
 
     loop_enabled = True if args.loop is not None else False
     docker_enabled = True if args.docker is not None else False
-    print(docker_enabled)
 
     return config_filename, export_formats, args.list_preferences, loop_enabled, docker_enabled
 
+
+def get_port(db_type):
+    if db_type == 'sql':
+        return '1433'
+    elif db_type == 'postgres':
+        return '5432'
+    elif db_type == 'mysql':
+        return '3308'
+    else:
+        return None
+
+
+def get_schema(db_type):
+    if db_type == 'sql':
+        return 'dbo'
+    elif db_type == 'postgres':
+        return 'public'
+    elif db_type == 'mysql':
+        return None
+    else:
+        return None
+
+
+def get_default_db(db_type):
+    if db_type == 'sql':
+        return 'iAuditor?driver=ODBC Driver 17 for SQL Server'
+    else:
+        return 'auditor'
+
+
+def ask_question(logger, str, q_type, choices=None, special=None, default=[]):
+    response = None
+    if q_type == 'multi':
+        response = questionary.select(str, default=default,
+                                      choices=choices
+                                      ).ask()
+    elif q_type == 'text':
+        response = questionary.text(str).ask()
+
+    elif q_type == 'int':
+        response = questionary.text(str).ask()
+        response = int(response)
+
+    elif q_type == 'bool':
+        response = questionary.confirm(str).ask()
+
+    if special == 'config_name':
+        response = sanitise_config_name(logger, response)
+
+    return response
+
+
+def sanitise_config_name(logger, config_name):
+    if config_name is None:
+        logger.info('The Config Name has been left blank, defaulting to iauditor.')
+        config_name = 'iauditor'
+    elif ' ' in config_name:
+        config_name = config_name.replace(' ', '_')
+    else:
+        config_name = config_name
+    if re.match("^[A-Za-z0-9_-]*$", config_name):
+        config_name = config_name
+
+    return config_name
+
+
+def setup_database(logger):
+    db_type = questionary.select("Which database type are you using?",
+                                 choices=[{'name': "SQL", 'value': 'mssql+pyodbc_mssql'},
+                                          {'name': "MySQL", 'value': 'mysql'},
+                                          {'name': "Postgres", 'value': 'postgres'}]
+                                 ).ask()
+    db_settings = {'database_type': db_type,
+                   'database_user': questionary.text("Database username").ask(),
+                   'database_pwd': questionary.password("Database Password").ask(),
+                   'database_server': questionary.text("Database Server Address").ask(),
+                   'database_port': ask_question(logger, 'Database Port', 'int', default=get_port(db_type)),
+                   'database_schema': questionary.text("Database Schema", default=get_schema(db_type)).ask(),
+                   'database_name': questionary.text("Database Name", default=get_default_db(db_type)).ask()}
+
+    return db_settings
+
+
+def box_print(h):
+    if h:
+        return Panel(h, box=box.HEAVY)
+    else:
+        return ''
+
+
+def update_key(logger, settings, key):
+    key_to_update = questions.get(key)
+    if key_to_update:
+        q = key_to_update['question']
+        t = key_to_update['type']
+        parent = key_to_update['parent']
+        if t == 'api_token':
+            new_value = sp.interactive_login()
+            if not new_value:
+                new_value = sp.interactive_login()
+        else:
+            if 'options' in key_to_update:
+                choices = key_to_update.get('options')
+            else:
+                choices = None
+            if parent:
+                current_value = settings[parent].get(key)
+            else:
+                current_value = settings.get(key)
+            h = f'''
+                Currently set to: {current_value} 
+                {key_to_update['header']}
+                '''
+            print(box_print(h))
+            new_value = ask_question(logger, q, t, choices, special=key_to_update)
+        if key_to_update['parent']:
+            settings[parent][key] = new_value
+        else:
+            settings[key] = new_value
+    elif key.startswith(('database_', 'sql_')):
+        db_settings = setup_database(logger)
+        print(db_settings)
+        sql_test = test_sql_settings(logger, db_settings)
+        if sql_test:
+            logger.info('Connected successfully, writing settings to config file.')
+            for k, v in db_settings.items():
+                settings['export_options'][k] = v
+        else:
+            logger.warning('Connection unsuccessful, trying again.')
+            setup_database()
+    else:
+        print(f'{key} is not a recognised setting. If unsure, start a new config file.')
+
+
+def modify_choice(logger, settings, config_path):
+    options = []
+    for k, v in settings.items():
+        options.append(Separator())
+        if k == 'config_name':
+            options.append(f'{k} - Current setting: {v}')
+        else:
+            for inner_k, inner_v in v.items():
+                if inner_k == 'sql_table':
+                    options.append(Separator())
+                if inner_k == 'token':
+                    inner_k = f'{inner_k} - (Required)'
+                options.append(f'{inner_k} - Current setting: {inner_v}')
+    print('Which setting do you need to modify?')
+    exit_no_save = 'Exit without saving changes'
+    exit_save = 'Exit and save'
+    if exit_no_save not in options:
+        exit_list = [exit_save, exit_no_save, Separator()]
+        options = exit_list + options
+    to_modify = questionary.select('Select the config option you want to change:', choices=options).ask()
+    to_modify = to_modify.split('-')[0].strip()
+    if to_modify == exit_save:
+        with open(config_path, 'w') as file:
+            updated_config = yaml.dump(settings, default_flow_style=False, sort_keys=False)
+            updated_config = updated_config.replace('-', '')
+            updated_config = updated_config.replace('null', '')
+            file.write(updated_config.replace('-', ''))
+        logger.info('All done. Re run the tool to use your new configuration. ')
+        sys.exit()
+    elif to_modify == exit_no_save:
+        logger.info('Changes discarded.')
+        sys.exit()
+    elif not to_modify:
+        modify_choice(logger, settings, config_path)
+    else:
+        update_key(logger, settings, to_modify)
+        modify_choice(logger, settings, config_path)
+
+
+def initial_setup(logger):
+    """
+    Creates a new directory in current working directory called 'iauditor_exports_folder'.  If 'iauditor_exports_folder'
+    already exists the setup script will notify user that the folder exists and exit. Default config file placed
+    in directory, with user API Token. User is asked for iAuditor credentials in order to generate their
+    API token.
+    :param logger:  the logger
+    """
+
+    # setup variables
+    current_directory_path = os.getcwd()
+    exports_folder_name = 'configs'
+
+    create_directory_if_not_exists(logger, exports_folder_name)
+
+    config_files = os.listdir(exports_folder_name)
+
+    if config_files:
+        if len(config_files) > 1:
+            config_to_edit = questionary.select(
+                "It looks like you already have some config files. Which one do you want to modify?",
+                choices=config_files
+            ).ask()
+        else:
+            config_to_edit = config_files[0]
+        config_path = os.path.join(exports_folder_name, config_to_edit)
+        with open(os.path.join(exports_folder_name, config_to_edit)) as file:
+            settings = yaml.load(file, Loader=yaml.FullLoader)
+    else:
+        settings = model_config
+        config_to_edit = 'config.yaml'
+        config_path = os.path.join(exports_folder_name, config_to_edit)
+    if 'config_name' in settings:
+        if settings['config_name']:
+            config_name = settings['config_name']
+        else:
+            config_name = 'iauditor'
+    else:
+        print('Your config file is missing config_name. Delete the file, '
+              're run the tool and it will be recreated correctly.')
+
+    current_ls = get_last_successful(logger, config_name)
+    parsed_date = parse_ls(current_ls)
+    if parsed_date:
+        pretty_file_name = parsed_date.strftime("%Y-%m-%d at %H%M")
+
+        update_ls_q = ask_question(logger, f'''We are currently searching for inspections completed after {pretty_file_name}, would you like to change this? Please note: if you're exporting with SQL, we'll grab this date by querying the database so changing this won't change anything for SQL exports.''', 'bool')
+        if update_ls_q:
+            print(box_print('''
+            You can use natural language here. For example if you wanted to start the search from March 22nd 2018, 
+            you could simply type 'March 22nd 2018'
+            '''))
+            change_ls = ask_question(logger, 'When should we search from? If you want to start from the beginning, '
+                                             'just press enter.', 'text')
+            new_date = parse_ls(change_ls)
+            if new_date:
+                new_date = new_date.strftime("%Y-%m-%dT%H:%M:%S%z")
+                update_sync_marker_file(str(new_date), config_name)
+
+    modify_choice(logger, settings, config_path)
+
+    sys.exit()
+    # # get token, set token
+    # token = sp.get_user_api_token(logger)
+    #
+    # if not token:
+    #     logger.critical("Problem generating API token.")
+    #     exit()
+    # DEFAULT_CONFIG_FILE_YAML[1] = '\n    token: ' + str(token)
+    #
+    # # create new directory
+    # create_directory_if_not_exists(logger, exports_folder_name)
+    #
+    # # write config file
+    # path_to_config_file = os.path.join(current_directory_path, exports_folder_name, 'config.yaml')
+    # if os.path.exists(path_to_config_file):
+    #     logger.critical("Config file already exists at {0}".format(path_to_config_file))
+    #     logger.info("Please remove or rename the existing config file, then retry this setup program.")
+    #     logger.info('Exiting.')
+    #     exit()
+    # try:
+    #     config_file = open(path_to_config_file, 'w')
+    #     config_file.writelines(DEFAULT_CONFIG_FILE_YAML)
+    # except Exception as ex:
+    #     log_critical_error(logger, ex, "Problem creating " + path_to_config_file)
+    #     logger.info("Exiting.")
+    #     exit()
+    # logger.info("Default config file successfully created at {0}.".format(path_to_config_file))
+    # os.chdir(exports_folder_name)
+    # choice = input('Would you like to start exporting audits from:\n  1. The beginning of time\n  '
+    #                '2. Now\n  Enter 1 or 2: ')
+    # if choice == '1':
+    #     logger.info('Audit exporting set to start from earliest audits available')
+    #     get_last_successful(logger)
+    # else:
+    #     now = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.000Z')
+    #     update_sync_marker_file(now)
+    #     logger.info('Audit exporting set to start from ' + now)
+    # exit()
